﻿package com.KameRoboAlarm;

import java.util.*;

import android.app.Activity;
import android.app.AlertDialog;
import android.os.Bundle;
import android.os.Build;

import android.app.AlarmManager;
import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.SharedPreferences.Editor;
import android.content.DialogInterface;

import android.util.Log;
import android.view.*;
import android.widget.Toast;

import com.robo.*;

/**
 * カメロボ、目覚まし時計
 */
public class KameRoboAlarm extends Activity implements Updatable {
	// JNI
	static {
		System.loadLibrary("roboctrlf");
	}

	private static final String COMMON_PREF = "CommonPref";
	private static final String PREF_MODE  = "Mode";
	private static final String PREF_TIME  = "Time";
	private static final String PREF_COUNT = "Count";
	private	static final int	SEC	= 1000;
	private	static final int	MIN = SEC*60;
	private	static final int	SNOOZE_TIME = 5;
	private	static final int	SOUND_STOP_TIME = 2*MIN;

	public	ClockImage			mClockImage;	// 時計ＵＩ
	public	WakeupImage			mWakeupImage;	// タイム起動の画面
	private AlarmManager		mManager;		// アラームマネージャ
	public static int			windowWidth;	// 画面横サイズ
	public static int			windowHeight;	// 画面縦サイズ

	private	Sound				soundAlarm;		// アラーム
	private	Sound				soundSe;		// SE

	private	boolean				mode;
	private	boolean				appliStop = false;

	private	int					touchCount;
	private	long				touchTime;
	private	SnoozeTh			snoozeTh;
	private	boolean				snoozeRun;
	private	int					sndReqCnt;
	private	SoundStopTh			soundStopTh;
	private	boolean				soundStopRun;

	/* add o2k 2011.03.04 */
	private boolean				bBeagle = false;
	private boolean				bRobo = false;
	private boolean				bComf = false;
	private boolean				bGlwf = false;

	private static final int   FPS           = 60;
	private static final long  TIME_TO_SLEEP = (long)(1000.0/FPS);

	private static String		brand  = Build.BRAND;
	private static String		device = Build.DEVICE;

	private int					jobCode;
	private int					oldCode;
	private int					subCode;

	private	long				ltime;
	private	long				rtime;
	private	long				timecnt;
	private float				rateTime;
	private	long				gtime;

	private final int 			JOB_COM_INIT   = 1;
	private final int 			JOB_RCB_RESET1 = 2;
	private final int 			JOB_RCB_RESET2 = 3;
	private final int 			JOB_GLW_INIT   = 4;
	private final int 			JOB_GLW_START1 = 5;
	private final int 			JOB_GLW_START2 = 6;
	private final int 			JOB_GLW_ACTION = 7;
	private final int 			JOB_NOR_WAIT   = 100;
	private final int 			JOB_ERR_CODE   = 200;

	// Handler for periodic update of screen.
	private UpdateHandler updateHandler = new UpdateHandler(this);

	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		WindowManager windowManager = getWindowManager();
		Display display = windowManager.getDefaultDisplay();
		windowWidth = display.getWidth();		// ウィンドウ横幅
		windowHeight = display.getHeight();		//	縦幅
		mManager = (AlarmManager) getSystemService(Context.ALARM_SERVICE);	// アラームマネージャを取得

		if (device.equals("beagleboard")){
			// 動作モード
			bBeagle = true;
		}

		mode = getMode(true);
		GregorianCalendar mCalendar = new GregorianCalendar();	// 現在時刻を取得
		long tm = mCalendar.getTimeInMillis();
		long wt = getTime(0);
		if ((wt-31000) <= tm && (wt+32000) >= tm && !mode){	// タイマー起動
			mWakeupImage = new WakeupImage(this);
			setContentView(mWakeupImage);
			soundAlarm = new Sound(this);
			soundAlarm.readRes(R.raw.alarm);
			soundAlarm.play(true);
			soundStopTh = new SoundStopTh();
			soundStopTh.start();
			sndReqCnt = getCount(0) + 1;
			setCount(sndReqCnt);
			bRobo = true;
		}
		else{									// タイマー設定モード
			if (0 != wt){
				Toast.makeText(this, "起動タイマーをキャンセルしました。", Toast.LENGTH_SHORT).show();
				resetAlarm();
			}
			mode = true;
			mClockImage = new ClockImage(this);	// 時計ＵＩ
			setContentView(mClockImage);		// 表示を時計ＵＩに移す
			mClockImage.setTime(mCalendar.get(Calendar.HOUR_OF_DAY), mCalendar.get(Calendar.MINUTE));
		}
		soundSe = new Sound(this);
		appliStop = false;
	}

	/**
	 * 	アラームの設定
	 * @param	time	アラーム時刻
	 */
	private void setAlarm(long time) {
		mManager.set(AlarmManager.RTC_WAKEUP, time, pendingIntent());
		setTime(time);
		mode = false;
		setMode(mode);
	}

	/**
	 * 	アラームの解除
	 */
	private void resetAlarm() {
		mManager.cancel(pendingIntent());
		setTime(0);
		mode = true;
		setMode(mode);
	}

	/**
	 * アラームの設定時刻に発生するインテントの作成
	 */
	private PendingIntent pendingIntent() {
		Intent i = new Intent(getApplicationContext(), KameRoboAlarm.class);
		PendingIntent pi = PendingIntent.getActivity(this, 0, i, 0);
		return pi;
	}

	/**
	 *	タッチイベント検出
	 */
	@Override
	public boolean onTouchEvent(MotionEvent event){
		if (appliStop) return false;				// 1.6でアプリ終了直後に呼ばれる事があるので対策する。

		int	act = event.getAction();
		if (mode){
			float touchX = event.getX();
			float touchY = event.getY();
			if (act == MotionEvent.ACTION_DOWN		// タッチした
				|| act == MotionEvent.ACTION_MOVE){	// 移動した
				mClockImage.setTouchPos(touchX, touchY, true);
			}
			else if (act == MotionEvent.ACTION_UP){		// 離した
				int r = mClockImage.setTouchPos(touchX, touchY, false);
				if (1 == r){						// セットボタン
					Calendar mCalendar = new GregorianCalendar();	// 現在時刻を取得
					Date trialTime = new Date();
					mCalendar.setTime(trialTime);
					int sh = mClockImage.getHours() % 12;
					int sm = mClockImage.getMinutes();
					int ch = mCalendar.get(Calendar.HOUR_OF_DAY) % 12;
					int cm = mCalendar.get(Calendar.MINUTE);
					int h = ((sh+24) - ch) % 12;
					int m = sm - cm;
//					h = 0;
//					m = 1;
					if (0 > m){
						m += 60;
						h--;
						if (0 > h) h += 12;
					}
					if (0 != h || 0 != m){
						String	awt = "";
						if (0 == h){
							awt = m+"分後";
						}
						else{
							awt = h+"時間"+m+"分後";
						}
						mCalendar.add(Calendar.HOUR, h);
						mCalendar.add(Calendar.MINUTE, m);
						String wt = ""+mCalendar.get(Calendar.HOUR_OF_DAY)+"時"+ mCalendar.get(Calendar.MINUTE)+"分";
						Toast.makeText(this, awt+"の"+wt+"に起動します。", Toast.LENGTH_LONG).show();
					}
					long tm = mCalendar.getTimeInMillis();
					setAlarm(tm);
					setCount(0);
					soundSe.readResPlay(R.raw.efct01, false);
					Log.d("MAIN","finish");
					appliStop = true;
					finish();
				}
				else if (2 == r){					// キャンセルボタン
					GregorianCalendar mCalendar = new GregorianCalendar();	// 現在時刻を取得
					mClockImage.setTime(mCalendar.get(Calendar.HOUR_OF_DAY), mCalendar.get(Calendar.MINUTE));
					soundSe.readResPlay(R.raw.efct02, false);
				}
				return true;
			}
		}
		else{
			if (act == MotionEvent.ACTION_DOWN){	// タッチした
				mWakeupImage.drawStop();
				setMode(true);
				soundAlarm.stop();
				soundSe.readResPlay(R.raw.efct03, false);
				long time =  System.currentTimeMillis();
				if (0 == touchCount || (touchTime+SEC) > time){
					touchCount++;
				}
				else{
					touchCount = 1;
				}
				touchTime = time;
				if (3 == touchCount || 4 <= sndReqCnt){
					snoozeRun = false;
					soundStopRun = false;
					setTime(0);
					finish();
				}
				else{
					if (null == snoozeTh){
						snoozeRun = true;
						snoozeTh = new SnoozeTh();
						snoozeTh.start();
					}
				}
			}
		}
		return false;
	}

	/**
	 * プリファレンスからモード値を読み出す
	 * @return	モード値
	 */
	private boolean getMode(boolean defMode){
		SharedPreferences activity = getPreferences(MODE_PRIVATE);
		return activity.getBoolean(PREF_MODE, defMode);
	}

	/**
	 * プリファレンスにモード値を書き出す
	 * @param	モード値
	 */
	private void setMode(boolean mode){
		SharedPreferences common = getSharedPreferences(COMMON_PREF, MODE_WORLD_READABLE | MODE_WORLD_WRITEABLE);
		Editor editor = common.edit();
		SharedPreferences activity = getPreferences(MODE_PRIVATE);
		editor = activity.edit();
		editor.putBoolean(PREF_MODE, mode);
		editor.commit();
	}

	/**
	 * プリファレンスからtime値を読み出す
	 * @return	time
	 */
	private long getTime(long defTime){
		SharedPreferences activity = getPreferences(MODE_PRIVATE);
		return activity.getLong(PREF_TIME, defTime);
	}

	/**
	 * プリファレンスにtime値を書き出す
	 * @param	time
	 */
	private void setTime(long time){
		SharedPreferences common = getSharedPreferences(COMMON_PREF, MODE_WORLD_READABLE | MODE_WORLD_WRITEABLE);
		Editor editor = common.edit();
		SharedPreferences activity = getPreferences(MODE_PRIVATE);
		editor = activity.edit();
		editor.putLong(PREF_TIME, time);
		editor.commit();
	}


	/**
	 * プリファレンスからCount値を読み出す
	 * @return	time
	 */
	private int getCount(int defCnt){
		SharedPreferences activity = getPreferences(MODE_PRIVATE);
		return activity.getInt(PREF_COUNT, defCnt);
	}

	/**
	 * プリファレンスにCount値を書き出す
	 * @param	time
	 */
	private void setCount(int cnt){
		SharedPreferences common = getSharedPreferences(COMMON_PREF, MODE_WORLD_READABLE | MODE_WORLD_WRITEABLE);
		Editor editor = common.edit();
		SharedPreferences activity = getPreferences(MODE_PRIVATE);
		editor = activity.edit();
		editor.putInt(PREF_COUNT, cnt);
		editor.commit();
	}

	/**
	 * Snooze
	 */
	class SnoozeTh extends Thread{
		private	int		tc;

		SnoozeTh(){
			super();
			tc = touchCount;
		}

		@Override
		public void run(){
			while(snoozeRun){
				long t = touchTime+SEC - System.currentTimeMillis();
				try {
					Thread.sleep(t);
				}
				catch(Exception e){
				}
				if (tc != touchCount){
					tc = touchCount;
				}
				else{
					Calendar mCalendar = new GregorianCalendar();	// 現在時刻を取得
					mCalendar.setTime(new Date());
					mCalendar.add(Calendar.MINUTE, SNOOZE_TIME);
					setAlarm(mCalendar.getTimeInMillis());
					appliStop = true;
					finish();
					break;
				}
			}
			snoozeTh = null;
		}
	}

	/**
	 * Sound stop
	 */
	class SoundStopTh extends Thread{
		SoundStopTh(){
			super();
			soundStopRun = true;
		}

		@Override
		public void run(){
			try {
				Thread.sleep(SOUND_STOP_TIME);
			}
			catch(Exception e){
			}
			if (soundStopRun){
				mWakeupImage.drawStop();
				setMode(true);
				soundAlarm.stop();
				if (3 == touchCount || 4 <= sndReqCnt){
					snoozeRun = false;
					soundStopRun = false;
					setTime(0);
					finish();
				}
				else{
					Calendar mCalendar = new GregorianCalendar();	// 現在時刻を取得
					mCalendar.setTime(new Date());
					mCalendar.add(Calendar.MINUTE, SNOOZE_TIME);
					setAlarm(mCalendar.getTimeInMillis());
					appliStop = true;
					finish();
				}
			}
			soundStopTh = null;
		}
	}

	@Override
	public void onResume()
	{
		super.onResume();

		if(bBeagle)
			update();

		// アクティビティがユーザとのやり取りを始められるようになる直前に呼ばれます。
		Log.v("Robo","onResume");
	}

	@Override
	public void onPause()
	{
		super.onPause();
		//アクティビティがバックグラウンドに隠れるときに呼ばれます。
		//例としては、現在の画面が他の画面を呼んだときなど。
		if(bBeagle)
			updateHandler.kill();

		Log.v("Robo","onPause");
	}

	// アクティビティが破棄される直前に呼ばれます。
	@Override
	public void onDestroy()
	{
		super.onDestroy();
		Log.d("MAIN","onDestroy");

		if(bGlwf){
			releaseGlwc();
			bGlwf = false;
		}

		if(bComf){
			SerialClosePort();
			bComf = false;
		}

		snoozeRun = false;
		soundStopRun = false;
		if (null != soundAlarm){
			soundAlarm.stop();
			soundAlarm.destroy();
		}
	}

	//	キーが押された
	@Override
	public boolean onKeyDown(int keyCode, KeyEvent event) {
		Log.i("main", "KEY ON  "+keyCode);
		switch (keyCode){
			case KeyEvent.KEYCODE_T:
				testSet();
			break;
			case KeyEvent.KEYCODE_D:
				if(jobCode==JOB_NOR_WAIT){
					bRobo = true;
					jobCode = JOB_RCB_RESET2;
				}
			break;
			default:
				return super.onKeyDown(keyCode,event);
		}

		return true;
	}

	public void update()
	{
		onExec();

		long etime = System.currentTimeMillis();

		long ntime = etime - ltime;

		ltime = etime;

		rtime += ntime;
		timecnt++;

		if(ntime>500) gtime = 500;
		else		  gtime = ntime;

		if(rtime>=1000){
			rateTime = (float)rtime / (float)timecnt ;
///			Log.i("robo","rate "+rateTime+"ms");
			rtime = 0;
			timecnt = 0;
		}

		updateHandler.sleep(TIME_TO_SLEEP);
	}

	private void onExec()
	{
		boolean bstart = false;

		if(jobCode!=oldCode){
			bstart = true;
			oldCode = jobCode;
			Log.d("Robo","job chenge [ " + jobCode + " ]");
		}

		switch(jobCode){
			case 0://
				jobCode = JOB_COM_INIT;
				break;
			case JOB_COM_INIT:		// COM
				if( SerialOpenPort("/dev/ttyUSB0", true) == true ) {
					bComf = true;
					jobCode = JOB_RCB_RESET1;
				} else {
					jobCode = JOB_ERR_CODE;
				}
				break;
			case JOB_RCB_RESET1:	// RCB-4HV 接続チェック
				resetCtrl();
				if(bRobo) jobCode = JOB_RCB_RESET2;
				else	  jobCode = JOB_NOR_WAIT;
				break;
			case JOB_RCB_RESET2:	// RCB-4HV 初期化
				serialMode(true);
				jobCode = JOB_GLW_INIT;
				break;
			case JOB_GLW_INIT:		// GLW 初期化
				initGlwc();
				bGlwf = true;
				jobCode = JOB_GLW_START1;
				break;
			case JOB_GLW_START1:	// モデル設定１
				startGlwc("sdcard/kame");
				jobCode = JOB_GLW_START2;
				break;
			case JOB_GLW_START2:	// モデル設定２
				surfaceGlwc(0,0);
				pauseSetGlwc(true);
				jobCode = JOB_GLW_ACTION;
				break;
			case JOB_GLW_ACTION:	// モーション再生
				drawGlwc();
				break;

			case JOB_ERR_CODE:
				if(bstart){
					showDialog(this,"error","NOT OPEN COM PORT");
				}
				break;
		}
	}

	private void testSet()
	{
		GregorianCalendar	mCalendar;		// 設定日時
		int hh =  0;
		int mm =  1;
		int ss =  0;

		if (0 != hh || 0 != mm || 0 != ss){
			if (0 == hh && 0 == mm){
				Toast.makeText(this, ""+ss+"秒後", Toast.LENGTH_SHORT).show();
			} else {
				if (0 == hh){
					Toast.makeText(this, ""+mm+"分後", Toast.LENGTH_SHORT).show();
				} else {
					Toast.makeText(this, ""+hh+"時間後"+mm+"分後", Toast.LENGTH_SHORT).show();
				}
			}
		}
		mCalendar = new GregorianCalendar();	// 現在時刻を取得
		long tm = mCalendar.getTimeInMillis() + hh*60*60*1000 + mm*60*1000 + ss*1000;
		Log.d("MAIN", "time ="+hh+" "+mm+" "+tm);
		setAlarm(tm);
		finish();

	}

	private native boolean 	SerialOpenPort(String portName, boolean debug);
	private native void 	SerialClosePort();

	private native boolean	resetCtrl();
	private native void		serialMode(boolean mode);

	private native void	 	initGlwc();
	private native void	 	releaseGlwc();
	private native void	 	startGlwc(String dataName);
	private native void  	surfaceGlwc(int w,int h);
	private native void  	drawGlwc();
	private native void  	pauseSetGlwc(boolean mode);

	private native int 		getErrorCode();

	// ダイアログ
	public static void showDialog(final Activity activity,String title,String text)
	{
		AlertDialog.Builder ad=new AlertDialog.Builder(activity);
		ad.setTitle(title);
		ad.setMessage(text);
		ad.setPositiveButton(
			"OK",new DialogInterface.OnClickListener()
			{
				public void onClick(DialogInterface dialog,int whichButton)
				{
					activity.setResult(Activity.RESULT_OK);
				}
			}
		);
		ad.create();
		ad.show();
	}
}
